<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2015 IMIA net based solutions (info@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaPageteaser\Controller;

use IMIA\ImiaBaseExt\Utility\Session\Storage\SessionStorage;
use TYPO3\CMS\Extbase\Mvc\View\ViewInterface;
use TYPO3\CMS\Core\Messaging\AbstractMessage;
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use IMIA\ImiaPageteaser\Domain\Repository\PageRepository;

/**
 * @package     imia_pageteaser
 * @subpackage  Controller
 * @author      David Frerich <d.frerich@imia.de>
 */
class BackendController extends \IMIA\ImiaBaseExt\Controller\ActionController
{
    /**
     * @var array
     */
    protected $pageData;

    /**
     * @var array
     */
    protected $tsConfig;

    /**
     * @var array
     */
    protected $extConf;

    /**
     * @var array
     */
    protected $moduleConf;

    /**
     * @var integer
     */
    protected $pageId;

    /**
     * @var SessionStorage
     */
    protected $sessionStorage;

    /**
     * @var PageRepository
     */
    protected $pageRepository;

    /**
     * @param SessionStorage $sessionStorage
     * @return AbstractController
     */
    public function injectSessionStorage(SessionStorage $sessionStorage)
    {
        $this->sessionStorage = $sessionStorage;
    }

    /**
     * @param PageRepository $pageRepository
     */
    public function injectPageRepository(PageRepository $pageRepository)
    {
        $this->pageRepository = $pageRepository;
    }

    /**
     * @param ViewInterface $view
     */
    public function initializeView(ViewInterface $view)
    {
        parent::initializeView($view);

        $this->controllerContext->getFlashMessageQueue()->getAllMessagesAndFlush();
        $this->extConf = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['imia_pageteaser']);

        $view->assign('controller', $this->getRequest()->getControllerName());
        $view->assign('action', $this->getRequest()->getControllerActionName());
        $view->assign('pageData', $this->getPageData());
        $view->assign('extConf', $this->extConf);
        $view->assign('moduleConf', $this->getModuleConfig());
        $view->assign('formToken', BackendUtility::getUrlToken('tceAction'));
    }

    public function indexAction()
    {
        if ($this->isInPage()) {
            if ($this->getParam('showHidden') ?: true) {
                $this->pageRepository->setIgnoreEnableFields(true);
                $this->view->assign('showHidden', $this->getParam('showHidden') ?: true);
            }

            $listFields = array_merge(
                array('title'),
                count($this->getModuleConfig('listFields')) > 0
                    ? array_keys($this->getModuleConfig('listFields'))
                    : array()
            );

            if ($this->getParam('sortingField') && in_array($this->getParam('sortingField'), $listFields)) {
                $sortingField = $this->getParam('sortingField');
                $sortingDirection = $this->getParam('sortingDirection') == 'desc' ? 'desc' : 'asc';
            } else {
                if (in_array('lastUpdated', $listFields)) {
                    $sortingField = 'lastUpdated';
                    $sortingDirection = 'desc';
                } else {
                    $sortingField = 'title';
                    $sortingDirection = 'asc';
                }
            }

            $sorting = array(
                $sortingField,
                $sortingDirection,
            );

            $language = $this->getParam('language') !== null ? (int)$this->getParam('language') : -1;
            if (!in_array($language, array_keys($this->getLanguages()))) {
                $language = -1;
            }
            $this->view->assign('language', $language);

            $page = (int)$this->getParam('page') ?: 1;
            if ($page <= 0) {
                $page = 0;
            }

            $perpage = (int)$this->getParam('perpage') ?: 20;
            if ($perpage <= 0) {
                $perpage = 20;
            }

            $offset = $perpage * $page  - $perpage;

            if ($this->getParam('recursive') ?: false) {
                $pageUids = $this->getRecursiveSubpages($this->getPageId());
            } else {
                $pageUids = array($this->getPageId());
            }
            $this->view->assign('recursive', $this->getParam('recursive') ?: false);

            if ($language >= 0) {
                $this->pageRepository->setLanguageUid($language);
            } else {
                $this->pageRepository->setShowAll(true);
            }

            $itemCount = $this->pageRepository->countByPidsAndDoktypesAndCategories(
                $pageUids,
                $this->getModuleConfig('doktypes'),
                null,
                null
            );
            $items = $this->pageRepository->findByPidsAndDoktypesAndCategories(
                $pageUids,
                $this->getModuleConfig('doktypes'),
                null,
                null,
                $perpage,
                $offset,
                $sorting
            );

            $this->view->assign('items', $items);
            $this->view->assign('pages', ceil($itemCount/$perpage));
            $this->view->assign('page', $page);
            $this->view->assign('perpage', $perpage);
            $this->view->assign('sortingField', $sortingField);
            $this->view->assign('sortingDirection', $sortingDirection);
        }
    }

    /**
     * @param integer $id
     * @param boolean $includeSelf
     * @return array
     */
    public function getRecursiveSubpages($id, $includeSelf = true)
    {
        $uids = array();
        if ($includeSelf) {
            $uids[] = (int)$id;
        }

        $res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid', 'pages', 'pid = ' . $id . ' AND deleted = 0');
        while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)) {
            $uids = array_merge($uids, $this->getRecursiveSubpages((int)$row['uid']));
        }

        return array_unique($uids);
    }

    /**
     * @return array
     */
    public function getLanguages()
    {
        $tsConfig = $this->getTSConfig();
        if (isset($tsConfig['mod.']['SHARED.']['defaultLanguageLabel'])) {
            $defaultFlag = $tsConfig['mod.']['SHARED.']['defaultLanguageFlag'];
            $defaultLabel = $tsConfig['mod.']['SHARED.']['defaultLanguageLabel'];
        } else {
            $defaultLabel = $this->translate('backend_language_default');
            $defaultFlag = '';
        }

        $languageOptions = array(
            -1  => $this->translate('backend_language_all'),
            0   => $defaultLabel
        );
        $languages = array(
            -1  => array(
                'uid'   => -1,
                'title' => $this->translate('backend_language_all'),
                'flag'  => 'multiple',
            ),
            0   => array(
                'uid'   => 0,
                'title' => $defaultLabel,
                'flag'  => $defaultFlag,
            ),
        );

        $result = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid, title, flag', 'sys_language');
        while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
            $languageOptions[$row['uid']] = $row['title'];
            $languages[$row['uid']] = $row;
        }

        $this->view->assign('languageOptions', $languageOptions);
        $this->view->assign('languages', $languages);

        return $languages;
    }

    /**
     * @param string $name
     * @return mixed
     */
    public function getParam($name)
    {
        $param = null;
        if ($this->request->hasArgument($name)) {
            $param = $this->request->getArgument($name);
            $this->sessionStorage->set('param_' . $name, $param);
        } elseif ($this->sessionStorage->has('param_' . $name)) {
            $param = $this->sessionStorage->get('param_' . $name);
        }

        return $param;
    }

    /**
     * @param string $key
     * @return mixed
     */
    public function getModuleConfig($key = null)
    {
        if (!$this->moduleConf) {
            $pluginNamespace = $this->getPluginNamespace();
            $module = (int)substr($pluginNamespace, strlen($pluginNamespace) - 1);

            $fieldData = $this->extConf['module' . $module . 'Fields'];
            $fields = array('l18n_cfg', 'l18n_cmd', 'create_language');
            foreach (preg_split('/[, ]+/ism', $fieldData) as $field) {
                if (trim($field)) {
                    $fields[] = trim($field);
                }
            }
            $fields = array_unique($fields);

            $tsConfig = BackendUtility::getTCEFORM_TSconfig('pages', $this->getPageData());
            $fieldListData = $this->extConf['module' . $module . 'FieldsList'];

            $fieldsList = array();
            foreach (preg_split('/[, ]+/ism', $fieldListData) as $field) {
                $field = trim($field);
                $label = $GLOBALS['TCA']['pages']['columns'][$field]['label'];
                $label = $tsConfig[$field]['label'] ?: $label;
                $label = $tsConfig[$field]['label.'][$GLOBALS['LANG']->lang] ?: $label;
                $label = $GLOBALS['LANG']->sL($label);
                if (strrpos($label, ':') === strlen($label)-1) {
                    $label = substr($label, 0, -1);
                }

                $fieldsList[$field] = $label;
            }

            /* $titleFieldLabel = $GLOBALS['TCA']['pages']['columns']['title']['label'];
            $titleFieldLabel = $tsConfig['title']['label'] ?: $titleFieldLabel;
            $titleFieldLabel = $tsConfig['title']['label.'][$GLOBALS['LANG']->lang] ?: $titleFieldLabel;
            $titleFieldLabel = $GLOBALS['LANG']->sL($titleFieldLabel);
            if (strrpos($titleFieldLabel, ':') === strlen($titleFieldLabel)-1) {
                $titleFieldLabel = substr($titleFieldLabel, 0, -1);
            }*/
            $titleFieldLabel = 'Titel';

            $this->view->assign('titleFieldLabel', $titleFieldLabel);

            $doktypeString = trim($this->extConf['module' . $module . 'Doktypes']);
            $basicDoktypes = $doktypeString ? array_map('intval', explode(',', $doktypeString)) : array();
            $doktypes = $basicDoktypes;
            foreach ($doktypes as $doktype) {
                $doktypes[] = $doktype + 3;
                $doktypes[] = $doktype + 4;
                $doktypes[] = $doktype + 6;

                if ($doktype == 110) {
                    $doktypes[] = $doktype + 1;
                    $doktypes[] = $doktype + 2;
                    $doktypes[] = $doktype + 5;
                }
            }

            $this->moduleConf = array(
                'title'         => $this->extConf['module' . $module . 'Title'],
                'description'   => $this->extConf['module' . $module . 'Description'],
                'doktypesBasic' => $basicDoktypes,
                'doktypes'      => $doktypes,
                'fields'        => $fields,
                'listFields'    => $fieldsList,
                'pid'           => $this->extConf['module' . $module . 'Pid'] ?: null,
            );
        }

        if ($key) {
            return $this->moduleConf[$key] ?: null;
        } else {
            return $this->moduleConf;
        }
    }

    /**
     * @return array
     */
    public function getTSConfig()
    {
        if (!$this->tsConfig) {
            $this->tsConfig = BackendUtility::getPagesTSconfig($this->getPageId());
        }

        return $this->tsConfig;
    }

    /**
     * @return boolean
     */
    public function isInPage()
    {
        $pageData = $this->getPageData();
        if ($pageData && $pageData['uid']) {
            return true;
        } else {
            $this->showInfo('nopage');
            return false;
        }
    }

    /**
     * @return integer
     */
    public function getPageId()
    {
        if (!$this->pageId) {
            $pluginNamespace = $this->getPluginNamespace();
            $module = (int)substr($pluginNamespace, strlen($pluginNamespace) - 1);
            $pid = $this->extConf['module' . $module . 'Pid'];

            if ($pid) {
                $this->pageId = $pid;
            } else {
                $this->pageId = (int)GeneralUtility::_GP('id');
            }
        }

        return $this->pageId;
    }

    /**
     * @param string $key
     * @return array|null
     */
    public function getPageData($key = null)
    {
        if (!$this->pageData) {
            $this->pageData = BackendUtility::getRecordWSOL('pages', $this->getPageId(), '*');
            if (!in_array((int)$this->pageData['doktype'], array(199, 254))) {
                $this->pageData['webView'] = true;
            }
        }

        if ($key && $this->pageData && is_array($this->pageData)) {
            return $this->pageData[$key];
        } else {
            return $this->pageData;
        }
    }

    /**
     * @param string $key
     * @param array $arguments
     */
    public function showInfo($key, $arguments = null)
    {
        $this->showFlashMessage($key, $arguments, AbstractMessage::INFO);
    }

    /**
     * @param string $key
     * @param array $arguments
     */
    public function showSuccess($key, $arguments = null)
    {
        $this->showFlashMessage($key, $arguments, AbstractMessage::OK);
    }

    /**
     * @param string $key
     * @param array $arguments
     */
    public function showWarning($key, $arguments = null)
    {
        $this->showFlashMessage($key, $arguments, AbstractMessage::WARNING);
    }

    /**
     * @param string $key
     * @param array $arguments
     */
    public function showFlashMessage($key, $arguments = null, $type = AbstractMessage::INFO)
    {
        $this->addFlashMessage(
            $this->translate('backend_' . $key, $arguments),
            null,
            $type,
            false
        );
    }
}