<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2015 IMIA net based solutions (info@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaPageteaser\Hook\Backend;

use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\MathUtility;

/**
 * @package     imia_pageteaser
 * @subpackage  Hook
 * @author      David Frerich <d.frerich@imia.de>
 */
class FormEngine
{
    /**
     * @var string
     */
    protected $typeShowitem;

    /**
     * @param string $table
     * @param string $row
     * @param \TYPO3\CMS\Backend\Form\FormEngine $formEngine
     */
    public function getMainFields_preProcess($table, $row, $formEngine)
    {
        $edit = GeneralUtility::_GP('edit');
        $overrideVals = GeneralUtility::_GP('overrideVals');
        $columnsOnlyAlt = GeneralUtility::_GP('columnsOnlyAlt');
        if ($columnsOnlyAlt && in_array($table, array_keys($edit))) {
            $typeNum = $formEngine->getRTypeNum($table, $row);
            $this->typeShowitem = $GLOBALS['TCA'][$table]['types'][$typeNum]['showitem'];
            $GLOBALS['TCA'][$table]['types'][$typeNum]['showitem'] = $columnsOnlyAlt;
        }

        $columnsOnlyTS = GeneralUtility::_GP('columnsOnlyTS');
        if ($columnsOnlyTS && in_array($table, array_keys($edit))) {
            $columns = array_map('trim', explode(',', $columnsOnlyTS));
            $formEngine->setTSconfig($table, $row);
            $key = $table . ':' . $row['uid'];

            foreach ($row as $field => $value) {
                $disabled = 1;
                if (in_array($field, $columns) || (is_array($overrideVals) && is_array($overrideVals[$table]) && in_array($field, array_keys($overrideVals[$table])))) {
                    $disabled = 0;
                }

                $formEngine->cachedTSconfig[$key][$field]['disabled'] = $disabled;
            }
        }
    }

    /**
     * @param string $content
     * @param array $record
     * @param string $table
     * @param array $parts
     * @param \TYPO3\CMS\Backend\Form\FormEngine $formEngine
     */
    public function getWrapTotal_preProcess(&$content, $record, $table, &$parts, $formEngine)
    {
        if (in_array($table, array('pages', 'pages_language_overlay')) && is_numeric($record['uid'])) {
            $pageLayoutEnabled = false;

            if (GeneralUtility::_GP('pagelayout')) {
                $pageLayoutEnabled = true;
            } else {
                $extConf = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['imia_pageteaser']);
                $pageTreeHiddenDoktypes = array_map('intval', explode(',', $extConf['doktypesPageTreeHidden']));
                if (in_array($record['doktype'], $pageTreeHiddenDoktypes)) {
                    $_GET['pagelayout'] = $_REQUEST['pagelayout'] = 1;
                    $pageLayoutEnabled = true;
                }
            }

            if ($pageLayoutEnabled) {
                $id = $table == 'pages' ? $record['uid'] : $record['pid'];

                if (in_array($record['doktype'], array(100, 110, 120, 130, 140))) {
                    if (ExtensionManagementUtility::isLoaded('templavoila')) {
                        $pagelayout = '';
                    } else {
                        $tmpSOBE = $GLOBALS['SOBE'];
                        $GLOBALS['SOBE'] = GeneralUtility::makeInstance('TYPO3\CMS\Backend\Controller\PageLayoutController');
                        $GLOBALS['SOBE']->doc = GeneralUtility::makeInstance('TYPO3\\CMS\\Backend\\Template\\DocumentTemplate');

                        $modName = 'web_layout';

                        $pageinfo = BackendUtility::readPageAccess($id, $GLOBALS['BE_USER']->getPagePermsClause(1));
                        $CALC_PERMS = $GLOBALS['BE_USER']->calcPerms($pageinfo);
                        $modSharedTSconfig = BackendUtility::getModTSconfig($id, 'mod.SHARED');
                        $modTSconfig = BackendUtility::getModTSconfig($id, 'mod.' . $modName);

                        $backendLayout = GeneralUtility::callUserFunction('TYPO3\CMS\Backend\View\BackendLayoutView->getSelectedBackendLayout', $id, $this);
                        if (count($backendLayout['__colPosList'])) {
                            $colPosList = implode(',', $backendLayout['__colPosList']);
                        }

                        $colPosList = array_unique(GeneralUtility::intExplode(',', $colPosList));
                        if (isset($modSharedTSconfig['properties']['colPos_list']) && trim($modSharedTSconfig['properties']['colPos_list']) !== '') {
                            $activeColPosList = array_unique(GeneralUtility::intExplode(',', trim($modSharedTSconfig['properties']['colPos_list'])));
                            if (!empty($colPosList) && !empty($activeColPosList)) {
                                $activeColPosList = array_unique(array_intersect(
                                    $activeColPosList,
                                    $colPosList
                                ));
                            }
                        } else {
                            $activeColPosList = $colPosList;
                        }
                        $activeColPosList = implode(',', $activeColPosList);
                        $colPosList = implode(',', $colPosList);

                        $dblist = GeneralUtility::makeInstance('TYPO3\CMS\Backend\View\PageLayoutView');
                        $dblist->backPath = $GLOBALS['BACK_PATH'];
                        $dblist->thumbs = GeneralUtility::_GP('imagemode');
                        $dblist->no_noWrap = 1;
                        $dblist->descrTable = $modName;
                        $dblist->script = 'db_layout.php';
                        $dblist->showIcon = 0;
                        $dblist->setLMargin = 0;

                        $dblist->doEdit = $CALC_PERMS & 16 ? 1 : 0;
                        $dblist->ext_CALC_PERMS = $CALC_PERMS;
                        $dblist->agePrefixes = $GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.xlf:labels.minutesHoursDaysYears');
                        $dblist->id = $id;

                        $dblist->nextThree = MathUtility::forceIntegerInRange($modTSconfig['properties']['editFieldsAtATime'], 0, 10);
                        $dblist->option_showBigButtons = $modTSconfig['properties']['disableBigButtons'] === '0';
                        $dblist->option_newWizard = $modTSconfig['properties']['disableNewContentElementWizard'] ? 0 : 1;
                        $dblist->defLangBinding = $modTSconfig['properties']['defLangBinding'] ? 1 : 0;
                        if (!$dblist->nextThree) {
                            $dblist->nextThree = 1;
                        }
                        $dblist->externalTables = $GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['cms']['db_layout']['addTables'];
                        $dblist->getTableMenu($id);

                        $tableOutput = array();
                        $tableJSOutput = array();
                        $CMcounter = 0;

                        foreach ($dblist->activeTables as $contentTable => $value) {
                            if (!isset($dblist->externalTables[$contentTable])) {
                                $dblist->tt_contentConfig['showCommands'] = 1;
                                $dblist->tt_contentConfig['showInfo'] = 1;
                                $dblist->tt_contentConfig['single'] = 0;

                                if (is_array($GLOBALS['TCA']['tt_content']['columns']['colPos']['config']['items'])) {
                                    $colList = array();
                                    $tcaItems = GeneralUtility::callUserFunction('TYPO3\\CMS\\Backend\\View\\BackendLayoutView->getColPosListItemsParsed', $id, $this);
                                    foreach ($tcaItems as $temp) {
                                        $colList[] = $temp[1];
                                    }
                                } else {
                                    $colList = array('1', '0', '2', '3');
                                }

                                if ($colPosList !== '') {
                                    $colList = array_intersect(GeneralUtility::intExplode(',', $colPosList), $colList);
                                }

                                $dblist->tt_contentConfig['cols'] = implode(',', $colList);
                                $dblist->tt_contentConfig['activeCols'] = $activeColPosList;
                                $dblist->tt_contentConfig['showHidden'] = 1;
                                $dblist->tt_contentConfig['sys_language_uid'] = $table == 'pages_language_overlay' ? $record['sys_language_uid'] : 0;
                            }

                            $dblist->itemsLimitSingleTable = 1000;
                            $dblist->start($id, $contentTable);
                            $dblist->counter = $CMcounter;
                            $dblist->ext_function = 1;

                            $dblist->generateList();

                            $tableOutput[$contentTable] = $dblist->HTMLcode;
                            $tableJSOutput[$contentTable] = $dblist->JScode;

                            $CMcounter += $dblist->counter;
                            $dblist->HTMLcode = '';
                            $dblist->JScode = '';
                        }

                        $pagelayout = '';
                        $headerContentHook = $GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['cms/layout/db_layout.php']['drawHeaderHook'];
                        if (is_array($headerContentHook)) {
                            foreach ($headerContentHook as $hook) {
                                $params = array();
                                $content .= GeneralUtility::callUserFunction($hook, $params, $this);
                            }
                        }

                        foreach ($tableOutput as $contentTable => $output) {
                            $pagelayout .= $GLOBALS['SOBE']->doc->spacer(15);
                            $pagelayout .= $GLOBALS['SOBE']->doc->section('', $output, true, true, 0, true);
                            $pagelayout .= $GLOBALS['SOBE']->doc->spacer(15);
                            $pagelayout .= $GLOBALS['SOBE']->doc->sectionEnd();
                        }

                        $footerContentHook = $GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['cms/layout/db_layout.php']['drawFooterHook'];
                        if (is_array($footerContentHook)) {
                            foreach ($footerContentHook as $hook) {
                                $params = array();
                                $content .= GeneralUtility::callUserFunction($hook, $params, $this);
                            }
                        }

                        $pagelayout = str_replace('db_new_content_el.php', '/typo3/sysext/cms/layout/db_new_content_el.php', $pagelayout);

                        $GLOBALS['SOBE'] = $tmpSOBE;
                    }
                } else {
                    $pagelayout = '';
                }

                $content .= $pagelayout . '<style type="text/css">body #typo3-inner-docbody > div {top: 20px;}' .
                    '.t3-gridContainer {margin: 0 -12px !important;} .t3-gridContainer > table {margin: 0}</style>';

                $tsConfig = BackendUtility::getPagesTSconfig($id);
                if (isset($tsConfig['mod.']['SHARED.']['defaultLanguageLabel'])) {
                    $defaultLabel = $tsConfig['mod.']['SHARED.']['defaultLanguageLabel'];
                } else {
                    $defaultLabel = $GLOBALS['LANG']->sL('LLL:EXT:imia_pageteaser/Resources/Private/Language/locallang.xlf:backend_language_default');
                }

                $page = $GLOBALS['TYPO3_DB']->exec_SELECTgetSingleRow('uid, l18n_cfg', 'pages', 'uid = ' . $id . ' AND deleted = 0');
                $disabled = '';
                if (in_array((int)$page['l18n_cfg'], array(1,3))) {
                    $disabled = ' ' . $GLOBALS['LANG']->sL('LLL:EXT:imia_pageteaser/Resources/Private/Language/locallang.xlf:backend_language_disabled');
                }

                $languages = array(
                    0 => array(
                        'uid'   => $id,
                        'label' => $defaultLabel . $disabled,
                        'table' => 'pages'
                    ),
                );

                $result = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid, title', 'sys_language');
                while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)) {
                    $pageOverlay = $GLOBALS['TYPO3_DB']->exec_SELECTgetSingleRow('uid', 'pages_language_overlay', 'pid = ' .
                        $id . ' AND sys_language_uid = ' . $row['uid'] . ' AND deleted = 0');

                    $languages[$row['uid']] = array(
                        'uid'   => $pageOverlay ? $pageOverlay['uid'] : null,
                        'label' => $row['title'],
                        'table' => 'pages_language_overlay'
                    );
                }

                $returnUrl = GeneralUtility::_GP('returnUrl');
                $columnsOnly = GeneralUtility::_GP('columnsOnly');
                $columnsOnlyAlt = GeneralUtility::_GP('columnsOnlyAlt');
                $columnsOnlyTS = GeneralUtility::_GP('columnsOnlyTS');

                $languageOptions = '';
                foreach ($languages as $langUid => $language) {
                    $params = array();

                    if ($language['uid']) {
                        $params[] = 'edit[' . $language['table'] . '][' . $language['uid'] . ']=edit';
                    } else {
                        $params[] = 'edit[' . $language['table'] . '][' . $id . ']=new&overrideVals[pages_language_overlay][doktype]=' .
                            $record['doktype'] . '&overrideVals[pages_language_overlay][sys_language_uid]=' . $langUid;

                        $language['label'] .= ' ' . $GLOBALS['LANG']->sL('LLL:EXT:imia_pageteaser/Resources/Private/Language/locallang.xlf:backend_language_new');
                    }

                    if ($columnsOnly) {
                        $params[] = 'columnsOnly=' . urlencode($columnsOnly);
                    }
                    if ($columnsOnlyAlt) {
                        $params[] = 'columnsOnlyAlt=' . urlencode($columnsOnlyAlt);
                    }
                    if ($columnsOnlyTS) {
                        $params[] = 'columnsOnlyTS=' . urlencode($columnsOnlyTS);
                    }
                    if ($returnUrl) {
                        $params[] = 'returnUrl=' . urlencode($returnUrl);
                    }
                    $params[] = 'pagelayout=1';

                    $selected = '';
                    if ($language['table'] == $table && $language['uid'] == $record['uid']) {
                        $selected = ' selected = "selected"';
                    }

                    $languageOptions .= '<option' . $selected . ' value="alt_doc.php?' . implode('&', $params) . '">' . $language['label'] . '</option>';
                }

                $parts[0] = '
                    <div style="float:right">
                        <label for="reclang">Sprache:</label>
                        <select id="reclang" name="reclang" onchange="location.href=this.value">' . $languageOptions . '</select>
                    </div>
                    <h1>' . $record['title'] . '&nbsp;</h1>
                    <div class="typo3-TCEforms">';

                if (ExtensionManagementUtility::isLoaded('imia_base')) {
                    $parts[1] .= \IMIA\ImiaBase\Xclass\Backend\View\PageLayoutView::getAdditionalStyles();
                }
            }
        }
    }

    /**
     * @param string $table
     * @param string $row
     * @param \TYPO3\CMS\Backend\Form\FormEngine $formEngine
     */
    public function getMainFields_postProcess($table, $row, $formEngine)
    {
        if ($this->typeShowitem) {
            $typeNum = $formEngine->getRTypeNum($table, $row);
            $GLOBALS['TCA'][$table]['types'][$typeNum]['showitem'] = $this->typeShowitem;
        }

        $columnsOnlyTS = GeneralUtility::_GP('columnsOnlyTS');
        if ($columnsOnlyTS) {
            unset($formEngine->cachedTSconfig);
        }
    }
}