<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2016 IMIA net based solutions (info@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaPageteaser\Xclass\Backend\Controller;

use TYPO3\CMS\Backend\Utility\BackendUtility;

/**
 * @package     imia_pageteaser
 * @subpackage  Xclass
 * @author      David Frerich <d.frerich@imia.de>
 */
class EditDocumentController extends \TYPO3\CMS\Backend\Controller\EditDocumentController
{
    /**
     * @inheritdoc
     */
    public function languageSwitch($table, $uid, $pid = null)
    {
        if (is_numeric($uid) && $this->getBackendUser()->check('tables_modify', $table)) {
            $languageField = $GLOBALS['TCA'][$table]['ctrl']['languageField'];
            $transForeignTable = $GLOBALS['TCA'][$table]['ctrl']['transForeignTable'];
            $transOrigPointerTable = $GLOBALS['TCA'][$table]['ctrl']['transOrigPointerTable'];
            $transOrigPointerField = $GLOBALS['TCA'][$table]['ctrl']['transOrigPointerField'];

            if ($transForeignTable && $this->getBackendUser()->check('tables_modify', $transForeignTable)) {
                $transForeignConfig = $GLOBALS['TCA'][$transForeignTable]['ctrl'];
                $transForeignLanguageField = $transForeignConfig['languageField'];

                if ($transForeignTable == 'pages_language_overlay') {
                    $pid = (int)$uid;
                } elseif (is_null($pid)) {
                    $row = BackendUtility::getRecord($table, $uid, 'pid');
                    $pid = (int)$row['pid'];
                }

                $langRows = $GLOBALS['TYPO3_DB']->exec_SELECTgetRows($transForeignTable . '.uid,' .
                    $transForeignTable . '.' . $transForeignLanguageField . ', sys_language.title',
                    $transForeignTable . ' LEFT JOIN sys_language ON sys_language.uid = ' . $transForeignTable . '.' . $transForeignLanguageField,
                    $transForeignTable . '.pid = ' . $pid . ' AND ' . $transForeignTable . '.deleted = 0');

                if ($langRows > 0) {
                    $languageMenu = $this->moduleTemplate->getDocHeaderComponent()->getMenuRegistry()->makeMenu();
                    $languageMenu->setIdentifier('_langSelector');
                    $languageMenu->setLabel($this->getLanguageService()->sL(
                        'LLL:EXT:lang/locallang_general.xlf:LGL.language',
                        true
                    ));

                    $tsConfig = BackendUtility::getPagesTSconfig($pid);
                    if (isset($tsConfig['mod.']['SHARED.']['defaultLanguageLabel'])) {
                        $defaultLabel = $tsConfig['mod.']['SHARED.']['defaultLanguageLabel'];
                    } else {
                        $defaultLabel = $this->translate('backend_language_default');
                    }

                    if ($this->getBackendUser()->checkLanguageAccess(0)) {
                        $href = BackendUtility::getModuleUrl('record_edit', array(
                            'edit[' . $table . '][' . $uid . ']' => 'edit',
                            'returnUrl'                          => $this->retUrl
                        ));
                        $menuItem = $languageMenu->makeMenuItem()
                            ->setTitle($defaultLabel)
                            ->setHref($href)
                            ->setActive(true);
                        $languageMenu->addMenuItem($menuItem);
                    }

                    foreach ($langRows as $langRow) {
                        if ($this->getBackendUser()->checkLanguageAccess($langRow[$transForeignLanguageField])) {
                            $href = BackendUtility::getModuleUrl('record_edit', array(
                                'edit[' . $transForeignTable . '][' . $langRow['uid'] . ']' => 'edit',
                                'returnUrl' => $this->retUrl
                            ));

                            $menuItem = $languageMenu->makeMenuItem()
                                ->setTitle($langRow['title'])
                                ->setHref($href);
                            $languageMenu->addMenuItem($menuItem);
                        }
                    }
                    $this->moduleTemplate->getDocHeaderComponent()->getMenuRegistry()->addMenu($languageMenu);
                }
            } elseif (is_numeric($uid) && $transOrigPointerTable && $transOrigPointerField && $this->getBackendUser()->check('tables_modify', $transOrigPointerTable)) {
                $row = BackendUtility::getRecord($table, $uid, $transOrigPointerField);

                 $langRows = $GLOBALS['TYPO3_DB']->exec_SELECTgetRows($table . '.uid,' .
                    $table . '.' . $languageField . ', sys_language.title',
                     $table . ' LEFT JOIN sys_language ON sys_language.uid = ' . $table . '.' . $languageField,
                     $table . '.pid = ' . $pid . ' AND ' . $table . '.deleted = 0');

                $languageMenu = $this->moduleTemplate->getDocHeaderComponent()->getMenuRegistry()->makeMenu();
                $languageMenu->setIdentifier('_langSelector');
                $languageMenu->setLabel($this->getLanguageService()->sL(
                    'LLL:EXT:lang/locallang_general.xlf:LGL.language',
                    true
                ));

                $tsConfig = BackendUtility::getPagesTSconfig($pid);
                if (isset($tsConfig['mod.']['SHARED.']['defaultLanguageLabel'])) {
                    $defaultLabel = $tsConfig['mod.']['SHARED.']['defaultLanguageLabel'];
                } else {
                    $defaultLabel = $this->translate('backend_language_default');
                }

                if ($this->getBackendUser()->checkLanguageAccess(0)) {
                    $transRecord = $GLOBALS['TYPO3_DB']->exec_SELECTgetSingleRow('uid',
                        $transOrigPointerTable, 'uid = ' . (int)$row[$transOrigPointerField]);

                    if ($transRecord) {
                        $href = BackendUtility::getModuleUrl('record_edit', array(
                            'edit[' . $transOrigPointerTable . '][' . $transRecord['uid'] . ']' => 'edit',
                            'returnUrl' => $this->retUrl
                        ));
                        $menuItem = $languageMenu->makeMenuItem()
                            ->setTitle($defaultLabel)
                            ->setHref($href);
                        $languageMenu->addMenuItem($menuItem);
                    }
                }

                foreach ($langRows as $langRow) {
                    if ($this->getBackendUser()->checkLanguageAccess($langRow[$languageField])) {
                        $href = BackendUtility::getModuleUrl('record_edit', array(
                            'edit[' . $transForeignTable . '][' . $langRow['uid'] . ']' => 'edit',
                            'returnUrl' => $this->retUrl
                        ));

                        $menuItem = $languageMenu->makeMenuItem()
                            ->setTitle($langRow['title'])
                            ->setHref($href);
                        if ($langRow['uid'] == $uid) {
                            $menuItem->setActive(true);
                        }
                        $languageMenu->addMenuItem($menuItem);
                    }
                }
                $this->moduleTemplate->getDocHeaderComponent()->getMenuRegistry()->addMenu($languageMenu);
            } else {
                parent::languageSwitch($table, $uid, $pid);
            }
        }
    }

    /**
     * @param string $key
     * @return string
     */
    protected function translate($key)
    {
        return $GLOBALS['LANG']->sL('LLL:EXT:imia_pageteaser/Resources/Private/Language/locallang.xlf:' . $key);
    }
}