<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (info@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaPageteaser\Xclass\Realurl;

use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;

if (!class_exists('tx_realurl_advanced')) {
	if (ExtensionManagementUtility::isLoaded('realurl')) {
		require_once(ExtensionManagementUtility::extPath('realurl', 'class.tx_realurl_advanced.php'));
	} else {
		require_once(ExtensionManagementUtility::extPath('imia_pageteaser', 'Classes/Dummy/Realurl/tx_realurl_advanced.php'));
	}
}

/**
 * Unique page-titles for url
 * http://forge.typo3.org/issues/54086
 *
 * @package     imia_base
 * @subpackage  Xclass
 * @author      David Frerich <d.frerich@imia.de>
 */
class Advanced extends \tx_realurl_advanced
{
    /**
     * @var array
     */
    protected $uniqueTitles = array();

    /**
     * @var \TYPO3\CMS\Frontend\Page\PageRepository
     */
    protected $pageRepository;

	/**
	 * @inheritdoc
	 */
    protected function rootLineToPath($rl, $lang)
    {
        $paths = array();
        array_shift($rl); // Ignore the first path, as this is the root of the website
        $c = count($rl);
        $stopUsingCache = false;
        $this->pObj->devLog('rootLineToPath starts searching', array('rootline size' => count($rl)));
        for ($i = 1; $i <= $c; $i++) {
            $page = array_shift($rl);

            // First, check for cached path of this page
            $cachedPagePath = false;
            if (!$page['tx_realurl_exclude'] && !$stopUsingCache && !$this->conf['disablePathCache']) {

                // Using pathq2 index!
                /** @noinspection PhpUndefinedMethodInspection */
                list($cachedPagePath) = $GLOBALS['TYPO3_DB']->exec_SELECTgetRows('pagepath', 'tx_realurl_pathcache',
                    'page_id=' . intval($page['uid']) .
                    ' AND language_id=' . intval($lang) .
                    ' AND rootpage_id=' . intval($this->conf['rootpage_id']) .
                    ' AND mpvar=' . $GLOBALS['TYPO3_DB']->fullQuoteStr($page['_MP_PARAM'], 'tx_realurl_pathcache') .
                    ' AND expire=0', '', '', 1);

                if (is_array($cachedPagePath)) {
                    $lastPath = implode('/', $paths);
                    $this->pObj->devLog('rootLineToPath found path', $lastPath);
                    if ($cachedPagePath != false && substr($cachedPagePath['pagepath'], 0, strlen($lastPath)) != $lastPath) {
                        // Oops. Cached path does not start from already generated path.
                        // It means that path was mapped from a parallel mount point.
                        // We cannot not rely on cache any more. Stop using it.
                        $cachedPagePath = false;
                        $stopUsingCache = true;
                        $this->pObj->devLog('rootLineToPath stops searching');
                    }
                }
            }

            // If a cached path was found for the page it will be inserted as the base of the new path, overriding anything build prior to this
            if ($cachedPagePath) {
                $paths = array();
                $paths[$i] = $cachedPagePath['pagepath'];
            }
            else {
                // Building up the path from page title etc.
                if (!$page['tx_realurl_exclude'] || count($rl) == 0) {
                    // List of "pages" fields to traverse for a "directory title" in the speaking URL (only from RootLine!!)
                    $segTitleFieldArray = $this->apiWrapper->trimExplode(',', $this->conf['segTitleFieldList'] ? $this->conf['segTitleFieldList'] : TX_REALURL_SEGTITLEFIELDLIST_DEFAULT, 1);
                    $theTitle = '';
                    foreach ($segTitleFieldArray as $fieldName) {
                        if (isset($page[$fieldName]) && $page[$fieldName] !== '') {
                            $theTitle = $page[$fieldName];
                            break;
                        }
                    }

                    /** ## BEGIN ADDED by dfrerich (UNIQUE TITLE) ### */
                    $theTitle = $this->getUniqueTitle($page, $fieldName);
                    /** ### END ADDED ### */

                    $paths[$i] = $this->encodeTitle($theTitle);
                }
            }
        }

        return implode('/', $paths);
    }

    /**
     * @param array $page
     * @param string $fieldName
     * @return string
     */
    protected function getUniqueTitle($page, $fieldName)
    {
		$languageUid = $this->pObj->getDetectedLanguage();
        $uniqueKey = $languageUid . '_' . $page['uid'];

        if (!array_key_exists($uniqueKey, $this->uniqueTitles)) {
            $titleUnique = true;
            $table = 'pages';

            if ($GLOBALS['TSFE'] && $GLOBALS['TSFE']->sys_page) {
                $enableFields = $GLOBALS['TSFE']->sys_page->enableFields($table);
            } else {
                $enableFields = BackendUtility::BEenableFields($table);
            }

            $pages = $GLOBALS['TYPO3_DB']->exec_SELECTgetRows('uid, pid, ' . $fieldName, $table,
                'pid = ' . (int)$page['pid'] . ' AND tx_realurl_exclude = 0' . $enableFields);

            foreach ($pages as $otherPage) {
                if ($languageUid > 0) {
                    $otherPage = $this->getPageRepository()->getPageOverlay($otherPage, $languageUid);
                }

                if ($otherPage['uid'] != $page['uid']) {
                    if ($this->encodeTitle($page[$fieldName]) == $this->encodeTitle($otherPage[$fieldName])) {
                        $titleUnique = false;
                        break;
                    }
                }
            }

            $this->uniqueTitles[$uniqueKey] = $page[$fieldName] . ($titleUnique ? '' : '-' . $page['uid']);
        }

        return $this->uniqueTitles[$uniqueKey];
    }

    /**
     * @return \TYPO3\CMS\Frontend\Page\PageRepository
     */
    protected function getPageRepository()
    {
        if (!$this->pageRepository) {
            $objectManager = GeneralUtility::makeInstance('TYPO3\CMS\Extbase\Object\ObjectManager');
            $this->pageRepository = $objectManager->get('TYPO3\CMS\Frontend\Page\PageRepository');
        }

        return $this->pageRepository;
    }

    /**
     * @inheritdoc
     */
	protected function findPageBySegmentAndPid($searchPid, $title)
    {
		// List of "pages" fields to traverse for a "directory title" in the speaking URL (only from RootLine!!)
		$segTitleFieldList = $this->conf['segTitleFieldList'] ? $this->conf['segTitleFieldList'] : TX_REALURL_SEGTITLEFIELDLIST_DEFAULT;
		$selList = $this->apiWrapper->uniqueList('uid,pid,doktype,mount_pid,mount_pid_ol,tx_realurl_exclude,' . $segTitleFieldList);
		$segTitleFieldArray = $this->apiWrapper->trimExplode(',', $segTitleFieldList, 1);

		// page select object - used to analyse mount points.
		$pageRepository = $this->apiWrapper->getPageRepository();

		// Build an array with encoded values from the segTitleFieldArray of the subpages
		// First we find field values from the default language
		// Pages are selected in menu order and if duplicate titles are found the first takes precedence!
		$titles = array(); // array(title => uid);
		$exclude = array();
		$uidTrack = array();
		/** @noinspection PhpUndefinedMethodInspection */
		$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery($selList, 'pages',
				'pid=' . intval($searchPid) .
				' AND deleted=0 AND doktype<>255', '', 'sorting');
		/** @noinspection PhpUndefinedMethodInspection */
		while (false != ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result))) {
			// Mount points
			$mount_info = $pageRepository->getMountPointInfo($row['uid'], $row);
			if (is_array($mount_info)) {
				// There is a valid mount point.
				if ($mount_info['overlay']) {
					// Overlay mode: Substitute WHOLE record
					/** @noinspection PhpUndefinedMethodInspection */
					$result2 = $GLOBALS['TYPO3_DB']->exec_SELECTquery($selList, 'pages',
							'uid=' . intval($mount_info['mount_pid']) .
							' AND deleted=0 AND doktype<>255');
					/** @noinspection PhpUndefinedMethodInspection */
					$mp_row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result2);
					if (is_array($mp_row)) {
						$row = $mp_row;
					}
					else {
						unset($row); // If the mount point could not be fetched, unset the row
					}
				}
				$row['_IS_MOUNTPOINT'] = $mount_info;
			}

			// Collect titles from selected row
			if (is_array($row)) {
				if ($row['tx_realurl_exclude']) {
					// segment is excluded
					$exclude[] = $row;
				}
				// Process titles. Note that excluded segments are also searched
				// otherwise they will never be found
				$uidTrack[$row['uid']] = $row;
				foreach ($segTitleFieldArray as $fieldName) {
					if (isset($row[$fieldName]) && $row[$fieldName] !== '') {
						$encodedTitle = $this->encodeTitle($row[$fieldName]);
						if (!isset($titles[$fieldName][$encodedTitle])) {
							$titles[$fieldName][$encodedTitle] = $row['uid'];
						}
					}
				}
			}
		}
		/** @noinspection PhpUndefinedMethodInspection */
		$GLOBALS['TYPO3_DB']->sql_free_result($result);

		// We have to search the language overlay too, if: a) the language isn't the default (0), b) if it's not set (-1)
		$uidTrackKeys = array_keys($uidTrack);
		$language = $this->pObj->getDetectedLanguage();
		if ($language != 0) {
			foreach ($uidTrackKeys as $l_id) {
				/** @noinspection PhpUndefinedMethodInspection */
				$result = $GLOBALS['TYPO3_DB']->exec_SELECTquery(TX_REALURL_SEGTITLEFIELDLIST_PLO,
						'pages_language_overlay',
						'pid=' . intval($l_id) . ' AND deleted=0' .
						($language > 0 ? ' AND sys_language_uid=' . $language : '')
				);
				/** @noinspection PhpUndefinedMethodInspection */
				while (false != ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result))) {
					foreach ($segTitleFieldArray as $fieldName) {
						if ($row[$fieldName]) {

                            /** ## BEGIN ADDED by dfrerich (UNIQUE TITLE) ### */
                            $uniqueTitle = $this->getUniqueTitle($row, $fieldName);
                            $encodedTitle = $this->encodeTitle($uniqueTitle);
                            /** ### END ADDED ### */

							//$encodedTitle = $this->encodeTitle($row[$fieldName]);
							if (!isset($titles[$fieldName][$encodedTitle])) {
								$titles[$fieldName][$encodedTitle] = $l_id;
							}
						}
					}
				}
				/** @noinspection PhpUndefinedMethodInspection */
				$GLOBALS['TYPO3_DB']->sql_free_result($result);
			}
		}

		// Merge titles
		$segTitleFieldArray = array_reverse($segTitleFieldArray); // To observe the priority order...
		$allTitles = array();
		foreach ($segTitleFieldArray as $fieldName) {
			if (is_array($titles[$fieldName])) {
				$allTitles = $this->apiWrapper->array_merge($allTitles, $titles[$fieldName]);
			}
		}

		// Return
		$encodedTitle = $this->encodeTitle($title);
		$possibleMatch = array();
		if (isset($allTitles[$encodedTitle])) {
			if (!$uidTrack[$allTitles[$encodedTitle]]['tx_realurl_exclude']) {
				return array($allTitles[$encodedTitle], $uidTrack[$allTitles[$encodedTitle]], false, array());
			}
			$possibleMatch = $uidTrack[$allTitles[$encodedTitle]];
		}
		return array(false, false, $exclude, $possibleMatch);
	}

    /**
     * @inheritdoc
     */
    protected function findIDBySegment($startPid, $mpvar, array &$urlParts, $currentIdMp = '', $foundUID = false)
    {
        // Creating currentIdMp variable if not set
        if (!is_array($currentIdMp)) {
            $currentIdMp = array($startPid, $mpvar, $foundUID);
        }

        // No more urlparts? Return what we have.
        if (count($urlParts) == 0) {
            return $currentIdMp;
        }

        // Get the title we need to find now
        $segment = array_shift($urlParts);

        // Perform search
        list($uid, $row, $exclude, $possibleMatch) = $this->findPageBySegmentAndPid($startPid, $segment);

        // If a title was found...
        if ($uid) {
            $newIdMp = $this->processFoundPage($row, $mpvar, $urlParts, true);
            if ($newIdMp[0] !== $uid || !in_array($row['doktype'], array(254, 199))) {
                return $newIdMp;
            }
        }
        elseif (count($exclude)) {
            // There were excluded pages, we have to process those!
            foreach ($exclude as $row) {
                $urlPartsCopy = $urlParts;
                array_unshift($urlPartsCopy, $segment);
                $result = $this->processFoundPage($row, $mpvar, $urlPartsCopy, false);
                if ($result[2]) {
                    $urlParts = $urlPartsCopy;
                    return $result;
                }
            }
        }

        // the possible "exclude in URL segment" match must be checked if no other results in
        // deeper tree branches were found, because we want to access this page also
        // + Books <-- excluded in URL (= possibleMatch)
        //   - TYPO3
        //   - ExtJS
        if (count($possibleMatch) > 0) {
            return $this->processFoundPage($possibleMatch, $mpvar, $urlParts, true);
        }

        // No title, so we reached the end of the id identifying part of the path and now put back the current non-matched title segment before we return the PID
        array_unshift($urlParts, $segment);
        
        return $currentIdMp;
    }
}